
function moms=RBF_moms(vertices,points,RBF_type,RBF_scale)

%--------------------------------------------------------------------------
% OBJECT.
% Moment cubature of RBF with centers points,  over the rectangle with
% vertices described by the vertices=[xv yv], using RBF described by
% "RBF_type" with common scale "RBF_scale".
%--------------------------------------------------------------------------
% INPUT
% vertices: it is a matrix N x 2, with components [xv yv] describing the
%           vertices of the rectangle
% RBF_type: 1=MQ, 2=G, 3=IMQ, 4=W2, 5=TPS
% RBF_scale: RBF scale.
%--------------------------------------------------------------------------
% OUTPUT
% moms: RBF moments, i.e int(phi(abs(x-t)/RBF_scale),rectangle)) where "t"
%       is the generic center and "phi" is the RBF.
%--------------------------------------------------------------------------
%--------------------------------------------------------------------------
% RELATED WORK.
%--------------------------------------------------------------------------
% Alvise Sommariva, Marco Vianello
% "Numerical cubature on scattered datas by Radial Basis Functions",
% Computing 76 (2005), 295-310.
%--------------------------------------------------------------------------
% Copyrights.
%--------------------------------------------------------------------------
%% Copyright (C) 2006-2019 Alvise Sommariva, Marco Vianello.
%%
%% This program is free software; you can redistribute it and/or modify
%% it under the terms of the GNU General Public License as published by
%% the Free Software Foundation; either version 2 of the License, or
%% (at your option) any later version.
%%
%% This program is distributed in the hope that it will be useful,
%% but WITHOUT ANY WARRANTY; without even the implied warranty of
%% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%% GNU General Public License for more details.
%%
%% You should have received a copy of the GNU General Public License
%% along with this program; if not, write to the Free Software
%% Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
%%
%% Author:  Alvise Sommariva <alvise@euler.math.unipd.it>
%%          Marco Vianello   <marcov@euler.math.unipd.it>
%%
%% Date: May 2004 - September 18, 2019.
%--------------------------------------------------------------------------

% SETTING DEFAULTS.

if nargin < 3
    RBF_type=4; % TPS.
end

if nargin < 4
    RBF_scale=1;
end

%--------------------------------------------------------------------------
% RBF Order: 1 for MQ, 2 for TPS, 0 otherwise
%--------------------------------------------------------------------------
switch RBF_type
    case 1
        RBF_order=1;
    case 5
        RBF_order=2;
    otherwise
        RBF_order=0;
end

if isscalar(RBF_scale) == 1
    RBF_scale=RBF_scale*ones(size(points,1),1);
end

Npts=size(points,1);

for ii=1:Npts
    center=points(ii,:);
    parms=[RBF_type RBF_scale(ii)];
    moms(ii,1)=RBF_mom_computation(center,vertices,parms);
end

% intfcn,N,polygon_sides,rotation,P,Q,quadf,cubature_type,varargin

[xyw,xvc,yvc,pgon,tri]=polygauss_2018(1,vertices(:,1),...
    vertices(:,2));
XX=xyw(:,1); YY=xyw(:,2); WW=xyw(:,3);

if RBF_order >= 1
    moms(Npts+1,1)=sum(WW);
end

if RBF_order >= 2
    moms(Npts+2,1)=XX'*WW;
    moms(Npts+3,1)=YY'*WW;
end



%++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
% ATTACHED SUBROUTINES
%++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

function mom=RBF_mom_computation(center,vertices,parms)

%--------------------------------------------------------------------------
% Descibe the polygon by its vertices, and fix the center.
%--------------------------------------------------------------------------

Nsides=size(vertices,1);

%--------------------------------------------------------------------------
% Compute the sides as vectors, and the relative measures.
%--------------------------------------------------------------------------
for i=1:Nsides
    if (i == Nsides)
        vector_side(i,:)=vertices(1,:)-vertices(Nsides,:);
        vector_side_mis(i,1)=norm(vector_side(Nsides,:),2);
    else
        vector_side(i,:)=vertices(i+1,:)-vertices(i,:);
        vector_side_mis(i,1)=norm(vector_side(i,:),2);
    end
    vector_center_vertex(i,:)=center-vertices(i,:);
    vector_center_vertex_mis(i,1)=norm(vector_center_vertex(i,:),2);
end

%--------------------------------------------------------------------------
% Compute the integral on each triangular subdivision of the polygon.
%--------------------------------------------------------------------------
for i=1:Nsides-1
    AO=vector_center_vertex(i,:)';
    BO=vector_center_vertex(i+1,:)';
    AB=vector_side(i,:)';
    misAO=vector_center_vertex_mis(i,1)';
    misBO=vector_center_vertex_mis(i+1,1)';
    misAB=vector_side_mis(i);
    VO=center';
    VA=vertices(i,:)';
    VB=vertices(i+1,:)';
    intTRIANGLE(i)=int_tr(AO,BO,AB,misAO,misBO,misAB, VO,VA,VB,parms);
end

AO=vector_center_vertex(Nsides,:)';
BO=vector_center_vertex(1,:)';
AB=vector_side(Nsides,:)';
misAO=vector_center_vertex_mis(Nsides,1)';
misBO=vector_center_vertex_mis(1,1)';
misAB=vector_side_mis(Nsides);
VO=center';
VA=vertices(Nsides,:)';
VB=vertices(1,:)';
intTRIANGLE(Nsides)=int_tr(AO,BO,AB,misAO,misBO,misAB, VO,VA,VB,parms);


%--------------------------------------------------------------------------
% Compute the integral on the polygon.
%--------------------------------------------------------------------------

mom=sum(intTRIANGLE);





function intOAB=int_tr(AO,BO,AB,misAO,misBO,misAB, VO,VA,VB,parms)

%--------------------------------------------------------------------------
% Input: VO, VA, VB: coordinates of the vertices of the triangle O,A,B,
%        where O is the center of the RBF. The triangle is general.
%        It will be divided in 2 rect. triangles:
%        let H be the projection of O on AB; then we divide OAB into OAH
%        and OHB. Finally we compute the integral on both triangles,
%        and sum the results to obtain the integral on OAB.
%--------------------------------------------------------------------------

%--------------------------------------------------------------------------
% Find H and compute the relative measures of the sides HA, HB, HO.
%--------------------------------------------------------------------------
versAB=(1/misAB)*AB;

scal_prA=AO'*versAB;
scal_prB=-BO'*versAB;

VH=VA+scal_prA*versAB;

HA=VA-VH;misHA=norm(HA,2);
HB=VB-VH;misHB=norm(HB,2);
HO=VO-VH;misHO=norm(HO,2);

%--------------------------------------------------------------------------
% Depending on the kind of triangle (acute or obtuse) OAB, we compute the
% integral, by subdivision in the 2 triangles rect. already computed.
%--------------------------------------------------------------------------

ortAB(1,1)=-AB(2,1);
ortAB(2,1)=AB(1,1);
scAO_ortAB=AO'*ortAB;

if (scAO_ortAB== 0)
    intOAB=0;
else
    
    if (scal_prA < 0)
        intOHB=int_tr_r(misHO,misBO,misHB,parms);
        intOHA=int_tr_r(misHO,misAO,misHA,parms);
        intOAB=intOHB-intOHA;
    end
    
    if (scal_prB < 0)
        intOHA=int_tr_r(misHO,misAO,misHA,parms);
        intOHB=int_tr_r(misHO,misBO,misHB,parms);
        intOAB=intOHA-intOHB;
    end
    
    if (scal_prA > 0) & (scal_prB > 0)
        intOHB=int_tr_r(misHO,misBO,misHB,parms);
        intOHA=int_tr_r(misHO,misAO,misHA,parms);
        intOAB=intOHA+intOHB;
    end
    
    if (scal_prA == 0)
        intOAB=int_tr_r(misAO,misBO, misAB,parms);
    end
    
    if (scal_prB == 0)
        intOAB=int_tr_r(misBO,misAO, misAB,parms);
    end
end





function y=int_tr_r(misHO,misBO,misHB,parms)

% The angle OAB is equal to "pi/2".

%--------------------------------------------------------------------------
% It computes the area of the rect. triangle OHB
%--------------------------------------------------------------------------

r0=misHO;
r1=misBO;

a=0; b=acos(r0/r1);

RBF_type=parms(1);
RBF_scale=parms(2);
parms(3)=r0;
tol=10^(-14);

    
% 1=MQ, 2=G, 3=IMQ, 4=W2, 5=TPS
switch RBF_type
    
    case 1
        %---------------------
        % MQ
        %---------------------
        parms=[RBF_type RBF_scale r0];
        [integral,f_evals] = quadl(@intfun,a,b,tol,0,parms);
        y=(RBF_scale^2)*integral;
        
    case 2
        %---------------------
        % GAUSSIAN
        %---------------------
        [integral,f_evals] = quadl(@intfun,a,b,tol,0,parms);
        y=(RBF_scale^2)*(0.5*b-0.5*integral);
        
    case 3
        %---------------------
        % IMQ
        %---------------------
        term1=-b;
        [integral,f_evals] =quadl(@intfun,a,b,tol,0,parms);
        term2=integral;
        y=term1+term2;
        y=(RBF_scale^2)*y;
        
    case 4
        %---------------------
        % COMPACT SUPPORT
        %---------------------
        
        %----------------------------------------------------------
        % KEY IDEA:
        % IN ORDER TO SIMPLIFY THE CODE, WE CONSIDER A TRIANGLE
        % THAT HAS THE SAME ANGLES BUT DIFFERENT SIZES DEPENDING
        % ON THE RBF SCALE. 
        % THIS ALLOWS US TO SIMPLIFY THE CODE, WORKING WITH A COMMON CODE
        % WITH SCALE EQUAL TO 1.
        %----------------------------------------------------------
        
        %----------------------------------------------------------
        % Theta is the angle of the triangle defined by r0 and r1.
        % Sigma is the support.
        %----------------------------------------------------------
        theta=b;
        r0=r0/RBF_scale;
        r1=r1/RBF_scale;
        
        parms(1)=RBF_type;
        parms(2)=1;
        parms(3)=r0;
        
        %--------------------------------------------
        % There are three cases, depending on the sup-
        % port and the scaling parameter.
        %--------------------------------------------
        if 1 <= min(r0,r1)
            %---------------------------------------------------
            % Support totally inside the triangle (sector part)
            %---------------------------------------------------
            angle_sigma=acos(r0);
            integral=intfun(angle_sigma,parms);
            y=theta*integral;
        else
            if 1 > max(r0,r1)
                %-------------------------------------------------
                % Support outside the triangle (triangular part)
                %-------------------------------------------------
                integral=W2_fastint(r0,r1,parms);
                y=integral;
            else
                %-------------------------------------
                % Support in part outside the triangle,
                % in part inside the triangle.
                %-------------------------------------
                
                theta_star=acos(r0);
                
                % Triangular part.
                integral=W2_fastint(r0,1,parms);
                
                y_triangular=integral;
                
                % Sector part
                angle_sigma=acos(r0);
                integral=intfun(angle_sigma,parms);
                y_sector=(theta-theta_star).*integral;
                
                y=y_triangular+y_sector;
            end
        end
        
        y=(RBF_scale^2)*y;
        
    case 5
        %---------------------
        %TPS
        %---------------------
        theta=acos(r0./r1);
        t=tan(theta/2);
        delta=RBF_scale;
        a1=r0./delta;
        t2=t.^2;
        t3=t.^3;
        t4=t.^4;
        t5=t.^5;
        t6=t.^6;
        term1=(a1.^4)./ ((1-t2).^3);
        term2=t.*(3-2*t2+3*t4);
        term3=log(a1.*(1+t2)./(1-t2));
        term4=33*t-38*t3+33*t5;
        term5=-2*(1-3*t2+3*t4-t6).*atan(1./t);
        y2=(1/6)*term1.*(term2.*term3-(1/12)*term4+term5);
        
        %variant, probably more stable, to compute "Ftheta"
        term6=(a1./cos(theta)).^3;
        term7=a1./((1+t2).^3);
        Ftheta=(1/6)*term6.*term7.*(term2.*term3-(1/12)*term4+term5);
        F0=-(1/6).*(a1.^4)*pi;
        y=Ftheta-F0;
        
        y=(RBF_scale^2)*y;
        
end





function y=intfun(x,parms)

RBF_type=parms(1);
RBF_scale=parms(2);
r0=parms(3);

bx=r0./(RBF_scale*cos(x));

% 1=MQ, 2=G, 3=IMQ, 4=W2, 5=TPS

switch RBF_type
    
    case 1
        % MQ
        y=((bx.^2 + 1).^(3/2))/3-((0^2 + 1)^(3/2))/3; 
        
    case 2
        %Gaussian
        y=exp(-bx.^2);
        
    case 3
        %IMQ
        y=sqrt(1+bx.^2);
        
    case 4
        
        %----------------------------------------------------------------------------------------
        % We compute here H_sigma(x)=\int_0^r(x) \phi(r/sigma)r dr where "sigma" is the support of
        % the radial basis. We notice that H_sigma(x)=sigma^2 \int_0^(r(x)/sigma) \phi(r)r dr.
        % For making the computations easier we make the transformation s=1-r, with ds=-dr.
        % In case 1 of variable w_type the analysis is done splitting the (rect.!) triangle, if needed,
        % in a triangular part and in a sector. In this case let "theta_int" the angle at the intersection
        % of the circle with radius "sigma" and the rect. triangle.
        %
        % PS: the input variable "x" is an angle!!!
        %
        % (TRIANGLE)   -
        %     ->      --
        %            ---
        %          -----X
        %         ------XX <- circle
        %        -------XXX
        %-----------------------------------------------------------------------------------------
        sigma=RBF_scale;
        
        r_sup=r0./cos(x);
        r_sup_div_sigma=r_sup/sigma;
        ext_sup=1-r_sup_div_sigma;
        ext_inf=1;
        
        wendland_fun=1;
        
        switch wendland_fun
            case 1
                term_sup=(ext_sup.^5)-(9/6)*(ext_sup.^6)+(4/7)*(ext_sup.^7);
                term_inf=(ext_inf.^5)-(9/6)*(ext_inf.^6)+(4/7)*(ext_inf.^7);
            case 2
                term_sup=(14/6)*(ext_sup.^6)-(35/7)*(ext_sup.^7)+(29/8)*(ext_sup.^8)-(8/9)*(ext_sup.^9);
                term_inf=(14/6)*(ext_inf.^6)-(35/7)*(ext_inf.^7)+(29/8)*(ext_inf.^8)-(8/9)*(ext_inf.^9);
            case 3
                term_sup=(66/9)*(ext_sup.^9)-(220/10)*(ext_sup.^10)+(275/11)*(ext_sup.^11)+(-153/12)*(ext_sup.^12)+(32/13)*(ext_sup.^13);
                term_inf=(66/9)*(ext_inf.^9)-(220/10)*(ext_inf.^10)+(275/11)*(ext_inf.^11)+(-153/12)*(ext_inf.^12)+(32/13)*(ext_inf.^13);
            case 4
                term_sup=(1/3)*(ext_sup.^3)-(1/4)*(ext_sup.^4);
                term_inf=(1/3)*(ext_inf.^3)-(1/4)*(ext_inf.^4);
            case 5
                term_sup=(56/7)*(ext_sup.^7)-(144/8)*(ext_sup.^8)+(123/9)*(ext_sup.^9)-(35/10)*(ext_sup.^10);
                term_inf=(56/7)*(ext_inf.^7)-(144/8)*(ext_inf.^8)+(123/9)*(ext_inf.^9)-(35/10)*(ext_inf.^10);
            case 6
                term_sup=(1/4)*(ext_sup.^4)-(1/5)*(ext_sup.^5);
                term_inf=(1/4)*(ext_inf.^4)-(1/5)*(ext_inf.^5);
            case 7
                term_sup=(ext_sup.^6)-(11/7)*(ext_sup.^7)+(5/8)*(ext_sup.^8);
                term_inf=(ext_inf.^6)-(11/7)*(ext_inf.^7)+(5/8)*(ext_inf.^8);
            case 8
                term_sup=(24/8)*(ext_sup.^8)-(63/9)*(ext_sup.^9)+(55/10)*(ext_sup.^10)-(16/11)*(ext_sup.^11);
                term_inf=(24/8)*(ext_inf.^8)-(63/9)*(ext_inf.^9)+(55/10)*(ext_inf.^10)-(16/11)*(ext_inf.^11);
            case 9
                term_sup=(1/2)*(ext_sup.^2)-(1/3)*(ext_sup.^3);
                term_inf=(1/2)*(ext_inf.^2)-(1/3)*(ext_inf.^3);
            case 10
                term_sup=(4/4)*(ext_sup.^4)-(7/5)*(ext_sup.^5)+(3/6)*(ext_sup.^6);
                term_inf=(4/4)*(ext_inf.^4)-(7/5)*(ext_inf.^5)+(3/6)*(ext_inf.^6);
        end
        
        y=-(sigma^2)*(term_sup-term_inf);
        
    case 5
        %TPS (eps=0!)
        y=(1/4)*(bx.^4).*log(bx+eps)-(1/16)*(bx.^4);
        
end




function tri_int=W2_fastint(r0,r1,parms)

RBF_scale=parms(2);

theta=acos(r0/r1);
a1=r0/RBF_scale;

%NB: The value of W2_int(a1,0) is 0.
tri_int=W2_int(a1,theta)-W2_int(a1,0);






function W2_integral=W2_int(a,t)

W2_integral=2/21*a^7/cos(t)^6*sin(t)+5/42*a^7/cos(t)^4*sin(t)+5/28*a^7/cos(t)^2*sin(t)+5/28*a^7*log(sec(t)+...
    tan(t))-1/2*a^6/cos(t)^5*sin(t)-2/3*a^6/cos(t)^3*sin(t)-4/3*a^6/cos(t)*sin(t)+a^5/cos(t)^4*sin(t)+...
    3/2*a^5/cos(t)^2*sin(t)+3/2*a^5*log(sec(t)+tan(t))-5/6*a^4/cos(t)^3*sin(t)-5/3*a^4/cos(t)*sin(t)+...
    1/2*a^2/cos(t)*sin(t);