
function weights=RBF_weights(vertices,points,RBF_type,RBF_scale)

%--------------------------------------------------------------------------
% OBJECT.
% Weights cubature of RBF with centers points,  over the rectangle with 
% vertices described by the vertices=[xv yv], using RBF described by 
% "RBF_type" with common scale "RBF_scale".
%--------------------------------------------------------------------------
% INPUT
% vertices: it is a matrix N x 2, with components [xv yv] describing the
%           vertices of the rectangle
% RBF_type: 1=MQ, 2=G, 3=IMQ, 4=W2, 5=TPS
% RBF_scale: RBF scale.
%--------------------------------------------------------------------------
% OUTPUT
% weights: RBF weights, relatively to pointset "points".
%--------------------------------------------------------------------------
%--------------------------------------------------------------------------
% RELATED WORK.
%--------------------------------------------------------------------------
% Alvise Sommariva, Marco Vianello
% "Numerical cubature on scattered datas by Radial Basis Functions", 
% Computing 76 (2005), 295-310. 
%--------------------------------------------------------------------------
% Copyrights.
%--------------------------------------------------------------------------
%% Copyright (C) 2006-2019 Alvise Sommariva, Marco Vianello.
%%
%% This program is free software; you can redistribute it and/or modify
%% it under the terms of the GNU General Public License as published by
%% the Free Software Foundation; either version 2 of the License, or
%% (at your option) any later version.
%%
%% This program is distributed in the hope that it will be useful,
%% but WITHOUT ANY WARRANTY; without even the implied warranty of
%% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%% GNU General Public License for more details.
%%
%% You should have received a copy of the GNU General Public License
%% along with this program; if not, write to the Free Software
%% Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
%%
%% Author:  Alvise Sommariva <alvise@euler.math.unipd.it>
%%          Marco Vianello   <marcov@euler.math.unipd.it>
%%
%% Date: May 2004 - September 14, 2019.
%--------------------------------------------------------------------------


%------------------------------------------------------------------
% Initial settings
%------------------------------------------------------------------

numpoints=size(points,1);
dim=size(points,2);

%------------------------------------------------------------------
% Unscaled RBF.
%------------------------------------------------------------------
phi=RBF(RBF_type);

%------------------------------------------------------------------
% RBF Order: 1 for MQ, 2 for TPS, 0 otherwise
%------------------------------------------------------------------
switch RBF_type
case 1
    RBF_order=1;
case 5
    RBF_order=2; 
otherwise
    RBF_order=0;
end

%------------------------------------------------------------------
% Computing RBF moments
%------------------------------------------------------------------
moms=RBF_moms(vertices,points,RBF_type,RBF_scale);

%------------------------------------------------------------------
% Computing interpolation matrix
%------------------------------------------------------------------
V = points2distances (points,points,RBF_scale);
V=phi(V'); % V is the INTEGRATION Vandermonde matrix!!

%------------------------------------------------------------------
% Computing weights.
%------------------------------------------------------------------
switch RBF_order
case 1
    V=[V  ones(numpoints,1) ; ones(1,numpoints) zeros(1,1)];
case 2
    V=[V  ones(numpoints,1)  points; ...
        ones(1,numpoints) zeros(1,1) zeros(1,dim); ...
        points' zeros(dim,1) zeros(dim,dim) ];
end

weights=V\moms;

switch RBF_order
case 1
    weights=weights(1:end-1);
case 2
    weights=weights(1:end-3);
end




